#include "i2c.h"
//-------------------
uChar *day_week[12] =
{
	(uChar*)"",
	(uChar*)"",
	(uChar*)"",
	(uChar*)"",
	(uChar*)"",
	(uChar*)"",
	(uChar*)""
};
//-------------
void I2C_Init()
{
	I2C_PORT = 0x00;
	I2C_DDR  = 0x03;
}
//----------------
void I2C_SDA_SET()
{
	I2C_DDR &= ~(1 << I2C_SDA);
	I2C_PORT &= ~(1 << I2C_SDA);
}
//------------------
void I2C_SDA_UNSET()
{
	I2C_DDR |= (1 << I2C_SDA);
	I2C_PORT &= ~(1 << I2C_SDA);
}
//----------------
void I2C_SCL_SET()
{
	I2C_DDR &= ~(1 << I2C_SCL);
	I2C_PORT &= ~(1 << I2C_SCL);
}
//------------------
void I2C_SCL_UNSET()
{
	I2C_DDR |= (1 << I2C_SCL);
	I2C_PORT &= ~(1 << I2C_SCL);
}
//--------------
void I2C_Start()
{
	I2C_SDA_SET();
	I2C_SCL_SET();
	_delay_us(I2C_DELAY);
	I2C_SDA_UNSET();
	_delay_us(I2C_DELAY);
	I2C_SCL_UNSET();
}
//-------------
void I2C_Stop()
{
	I2C_SDA_UNSET();
	I2C_SCL_UNSET();
	_delay_us(I2C_DELAY);
	I2C_SCL_SET();
	_delay_us(I2C_DELAY);
	I2C_SDA_SET();
	_delay_us(I2C_DELAY);
}
//-------------------------
uChar I2C_Write(uChar byte)
{
	uChar bit;
	uChar ask;
	
	for(uChar count = 0; count < 8; count++)
	{
		bit = byte;
		
		bit <<= count;
		bit &= 0x80;
		
		I2C_SCL_UNSET();
		_delay_us(I2C_DELAY);
		
		if(!bit)             //  bit = 0 
			I2C_SDA_UNSET(); //  sda = 0, 
		else
			I2C_SDA_SET(); //  sda = 1
		
		_delay_us(I2C_DELAY);
			
		I2C_SCL_SET();
		_delay_us(I2C_DELAY);
	}
	
	I2C_SCL_UNSET();
	_delay_us(I2C_DELAY);
	
	I2C_DDR &= ~(1 << I2C_SDA); //   sda     ask
	
	I2C_SCL_SET();
	_delay_us(I2C_DELAY);
	
	if(!(I2C_PIN & (1 << I2C_SDA)))
		ask = I2C_ASK;
	else
		ask = I2C_NASK;
		
	I2C_SCL_UNSET();
	
	I2C_DDR |= (1 << I2C_SDA); //   sda 
	
	return ask;
}
//-----------------------
uChar I2C_Read(uChar ask)
{
	uChar byte = 0; //  
	
	I2C_DDR &= ~(1 << I2C_SDA); //   sda 
	
	for(uChar count = 8; count > 0; count--)
	{
		I2C_SCL_SET();
		_delay_us(I2C_DELAY);
		
		if((I2C_PIN & (1 << I2C_SDA)))	
			byte |= (1 << (count - 1));
			
		I2C_SCL_UNSET();
		_delay_us(I2C_DELAY);
	}
	
	I2C_SCL_UNSET();
	_delay_us(I2C_DELAY);
	
	if(ask == I2C_ASK)
		I2C_SDA_UNSET();
	else
		I2C_SDA_SET();
		
	_delay_us(I2C_DELAY);
	
	I2C_SCL_SET();
	_delay_us(I2C_DELAY);
	I2C_SCL_UNSET();
	_delay_us(I2C_DELAY);
	
	return byte;
}
//-----------------------------------
uChar I2C_DecodeToDecimal(uChar byte)
{
	uChar result = ((byte >> 4)*10 + (byte & 0x0F));
	
	return result;
}
//--------------
uChar I2C_Hour()
{
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_HOUR);
	
	I2C_Start();
	I2C_Write(ADDR_DS1307_READ);
	uChar hour = I2C_Read(I2C_NASK);
	I2C_Stop();
	
	return hour;
}
//----------------
uChar I2C_Minute()
{
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_MINUTE);
	
	I2C_Start();
	I2C_Write(ADDR_DS1307_READ);
	uChar minute = I2C_Read(I2C_NASK);
	I2C_Stop();
	
	return minute;
}
//----------------
uChar I2C_Second()
{
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_SECOND);
	
	I2C_Start();
	I2C_Write(ADDR_DS1307_READ);
	uChar second = I2C_Read(I2C_NASK);
	I2C_Stop();
	
	return second;
}
//-------------
uChar I2C_Day()
{
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_DAY);
	
	I2C_Start();
	I2C_Write(ADDR_DS1307_READ);
	uChar date = I2C_Read(I2C_NASK);
	I2C_Stop();
	
	return date;
}
//---------------
uChar I2C_Month()
{
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_MONTH);
	
	I2C_Start();
	I2C_Write(ADDR_DS1307_READ);
	uChar month = I2C_Read(I2C_NASK);
	I2C_Stop();
	
	return month;
}
//--------------
uChar I2C_Year()
{
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_YEAR);
	
	I2C_Start();
	I2C_Write(ADDR_DS1307_READ);
	uChar year = I2C_Read(I2C_NASK);
	I2C_Stop();
	
	return year;
}
//-----------------
uChar I2C_DayWeek()
{
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_WEEK_DAY);
	
	I2C_Start();
	I2C_Write(ADDR_DS1307_READ);
	uChar week_day = I2C_Read(I2C_NASK);
	I2C_Stop();
	
	return week_day;
}
//------------------------
void I2C_Time(uChar *time)
{
	uChar hour   = I2C_Hour();
	uChar minute = I2C_Minute();
	uChar second = I2C_Second();
	
	hour   = I2C_DecodeToDecimal(hour);
	minute = I2C_DecodeToDecimal(minute);
	second = I2C_DecodeToDecimal(second);
	
	uChar buffer[20];
	sprintf((char*)buffer, "%d", hour);
	if(hour < 10)
	{
		strcpy((char*)time, "0");
		strcat((char*)time, (char*)buffer);
	}
	else
		strcpy((char*)time, (char*)buffer);
		
	strcat((char*)time, ":");
	
	if(minute < 10)
		strcat((char*)time, "0");
	sprintf((char*)buffer, "%d", minute);
	strcat((char*)time, (char*)buffer);
	strcat((char*)time, ":");
	if(second < 10)
		strcat((char*)time, "0");
	sprintf((char*)buffer, "%d", second);
	strcat((char*)time, (char*)buffer);
}
//------------------------
void I2C_Date(uChar *date)
{
	uChar day      = I2C_Day();
	uChar month    = I2C_Month();
	uChar year     = I2C_Year();
	uChar week_day = I2C_DayWeek();
	
	day      = I2C_DecodeToDecimal(day);
	month    = I2C_DecodeToDecimal(month);
	year     = I2C_DecodeToDecimal(year);
	week_day = I2C_DecodeToDecimal(week_day);
	
	uChar buffer[20];
	sprintf((char*)buffer, "%d", day);
	if(day < 10)
	{
		strcpy((char*)date, "0");
		strcat((char*)date, (char*)buffer);
	}
	else
		strcpy((char*)date, (char*)buffer);
		
	strcat((char*)date, ".");
	
	sprintf((char*)buffer, "%d", month);
	if(month < 10)
		strcat((char*)date, "0");
	strcat((char*)date, (char*)buffer);
	strcat((char*)date, ".");
	
	sprintf((char*)buffer, "%d", year);
	if(year < 10)
		strcat((char*)date, "0");
	strcat((char*)date, (char*)buffer);
	
	strcat((char*)date, ", ");
	strcat((char*)date, (char*)day_week[week_day - 1]);
}
//--------------------
void I2C_DS1307_Init()
{
	uChar set = I2C_Hour();
	
	if(set & 0x4F)
		set = (set & 0x3F);
		
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_HOUR);
	I2C_Write(set);
	I2C_Stop();
	
	set = I2C_Second();
	
	if(set & 0x80)
		set = (set & 0x7F);
		
	I2C_Start();
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_SECOND);
	I2C_Write(set);
	I2C_Stop();
}
//------------------------------------------------------
void I2C_SetTime(uChar hour, uChar minute, uChar second)
{
	I2C_Start();
	
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_SECOND);
	second = I2C_DecodeToBinDec(second);
	I2C_Write(second);
	
	I2C_Stop();
	
	I2C_Start();
	
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_MINUTE);
	minute = I2C_DecodeToBinDec(minute);
	I2C_Write(minute);
	
	I2C_Stop();
	
	I2C_Start();
	
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_HOUR);
	hour = I2C_DecodeToBinDec(hour);
	I2C_Write(hour);
	
	I2C_Stop();
}
//------------------------------------------------------------------
void I2C_SetDate(uChar day, uChar month, uChar year, uChar week_day)
{
	I2C_Start();
	
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_DAY);
	day = I2C_DecodeToBinDec(day);
	I2C_Write(day);
	
	I2C_Stop();
	
	I2C_Start();
	
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_MONTH);
	month = I2C_DecodeToBinDec(month);
	I2C_Write(month);
	
	I2C_Stop();
	
	I2C_Start();
	
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_YEAR);
	year = I2C_DecodeToBinDec(year);
	I2C_Write(year);
	
	I2C_Stop();
	
	I2C_Start();
	
	I2C_Write(ADDR_DS1307_WRITE);
	I2C_Write(ADDR_DS1307_WEEK_DAY);
	week_day = I2C_DecodeToBinDec(week_day);
	I2C_Write(week_day);
	
	I2C_Stop();
}
//----------------------------------
uChar I2C_DecodeToBinDec(uChar byte)
{
	uChar high = byte/10;
	uChar low  = byte - (high*10);
	
	return ((high << 4) | low);
}