#include "lcd.h"
//-------------------------------------
//--------- -------
uChar EDITSHOW = 0x0C; //   ( ,  )
uChar ALIGMENT = LCD_ALIGMENT_NONE; //   (  -  )
//-------------------------------------
void LCD_Command(uChar command)
{
	LCD_PORT &= ~(1 << LCD_RS); // RS = 0 -  
	uChar temp = (command & 0xF0); //     +  
	temp |= (LCD_PORT & 0x0F); 
	LCD_PORT = temp;
	LCD_PORT |= (1 << LCD_E); // E = 1
	_delay_us(40);
	LCD_PORT &= ~(1 << LCD_E); // E = 0
	temp = (command << 4); //     
	temp |= (LCD_PORT & 0x0F);
	LCD_PORT = temp;
	LCD_PORT |= (1 << LCD_E); // E = 1
	_delay_us(40);
	LCD_PORT &= ~(1 << LCD_E); // E = 0
	_delay_us(100);
}
//-------------------------------
void LCD_Data(uChar data)
{
	LCD_PORT |= (1 << LCD_RS); // RS = 1 -  
	uChar temp = (data & 0xF0); //     +  
	temp |= (LCD_PORT & 0x0F); 
	LCD_PORT = temp;
	LCD_PORT |= (1 << LCD_E); // E = 1
	_delay_us(40);
	LCD_PORT &= ~(1 << LCD_E); // E = 0
	temp = (data << 4); //     
	temp |= (LCD_PORT & 0x0F);
	LCD_PORT = temp;
	LCD_PORT |= (1 << LCD_E); // E = 1
	_delay_us(40);
	LCD_PORT &= ~(1 << LCD_E); // E = 0
	_delay_us(100);
}
//-----------------
void LCD_Init(void)
{
	LCD_DDR  = 0xFE; //   
	LCD_PORT = 0x00; //     

	LCD_Command(0x03);
	_delay_us(40);
	LCD_Command(0x03);
	_delay_us(40);
	LCD_Command(0x03);
	_delay_us(40);
	LCD_Command(0x02); 
	_delay_us(40);
	LCD_Command(0x28); 
	_delay_us(40);
	LCD_Command(0x08); 
	_delay_us(40);
	LCD_Command(0x0C);
	_delay_us(40);
	LCD_Command(0x01); 
	_delay_us(1500);
}
/***********************************************************************
*                                        *
***********************************************************************/
uChar LCD_CharToCode(uChar c)
{
	switch(c)
	{
			case '' : return (0x41);
			case '' : return (0xa0);
			case '' : return (0x42);
			case '' : return (0xa1);
			case '' : return (0xe0);
			case '' : return (0x45);
			case '' : return (0xa3);
			case '' : return (0xa4);
			case '' : return (0xa5);
			case '' : return (0xa6);
			case '' : return (0x4b);
			case '' : return (0xa7);
			case '' : return (0x4d);
			case '' : return (0x48);
			case '' : return (0x4F);
			case '' : return (0xa8);
			case '' : return (0x50);
			case '' : return (0x43);
			case '' : return (0x54);
			case '' : return (0xa9);
			case '' : return (0xaa);
			case '' : return (0x58);
			case '' : return (0xe1);
			case '' : return (0xab);
			case '' : return (0xac);
			case '' : return (0xe2);
			case '' : return (0xae);
			case '' : return (0xaf);
			case '' : return (0xb0);
			case '' : return (0xb1);
			case '' : return (0x61);        
			case '' : return (0xb2);
			case '' : return (0xb3);
			case '' : return (0xb4);
			case '' : return (0xe3);        
			case '' : return (0x65);
			case '' : return (0xb5);
			case '' : return (0xb6);
			case '' : return (0xb7);
			case '' : return (0xb8);        
			case '' : return (0xb9);
			case '' : return (0xba);
			case '' : return (0xbb);
			case '' : return (0xbc);        
			case '' : return (0xbd);
			case '' : return (0x6f);
			case '' : return (0xbe);
			case '' : return (0x70);        
			case '' : return (0x63);
			case '' : return (0xbf);
			case '' : return (0x79);
			case '' : return (0xe4);
			case '' : return (0x78);
			case '' : return (0xe5);
			case '' : return (0xc0);
			case '' : return (0xc1);
			case '' : return (0xe6);
			case '' : return (0xc2);
			case '' : return (0xc3);
			case '' : return (0xc4);
			case '' : return (0xc5);
			case '' : return (0xc6);
			case '' : return (0xc7);
	}
	
	return c;
} 
//-----------------------------------------
void LCD_PrintString(uChar *str, uChar row)
{
	uChar lenght = strlen((char*)str);
	uChar position;
	
	if((position = LCD_Position(lenght)) != 255)
		LCD_Goto(position, row);
	else
		LCD_Goto(0, row);
	
	for(uChar index = 0; index < lenght; index++)
	{
		uChar c = LCD_CharToCode(*str++);
		LCD_Data(c);
	}		
}
//---------------------------------------------
void LCD_PrintSymbol(uChar codeChar, uChar row)
{
	LCD_Command(row);
	LCD_Data(codeChar);
}
//-----------------------------
void LCD_Goto(uChar x, uChar y)
{
	uChar pos = x + y;
	
	LCD_Command(pos);
	_delay_us(40);
}
//--------------
void LCD_Clear()
{
	LCD_Command(0x01);
	_delay_ms(10);
}
//-------------------------
void LCD_Cursor(bool state)
{
	if(state)
		EDITSHOW |= (1 << LCD_CURSOR_STATE);
	else
		EDITSHOW &= ~(1 << LCD_CURSOR_STATE);
		
	LCD_Command(EDITSHOW);
}
//------------------------------
void LCD_CursorBlink(bool state)
{
	if(state)
		EDITSHOW |= (1 << LCD_CURSOR_FLASH);
	else
		EDITSHOW &= ~(1 << LCD_CURSOR_FLASH);
	
	LCD_Command(EDITSHOW);
}
//-----------------------------------------------
void LCD_SaveUserSign(uChar *sign, uChar address)
{
	LCD_Command(address);
	
	for(uChar index = 0; index < 8; index++)
	LCD_Data(sign[index]);
}	
//----------------------------------
void LCD_SetAligment(uChar aligment)
{
	ALIGMENT = aligment;
}
//------------------------------
uChar LCD_Position(uChar lenght)
{
	uChar position;
	
	switch(ALIGMENT)
	{
		case LCD_ALIGMENT_NONE:
			position = 255;
		break;
		
		case LCD_ALIGMENT_LEFT:
			position = 0;
		break;
		
		case LCD_ALIGMENT_RIGHT:
			position = LCD_MAX_LINE_SIZE - lenght;
		break;
			
		case LCD_ALIGMENT_CENTER:
			position = (LCD_MAX_LINE_SIZE - lenght)/2;
		break;
			
		default: position = 255; break;
	}
	
	return position;
}