#include "onewire.h"
//----------------
void ONEWIRE_SET()
{
	ONEWIRE_DDR  &= ~(1 << ONEWIRE_DQ);
	ONEWIRE_PORT &= ~(1 << ONEWIRE_DQ);
}
//------------------
void ONEWIRE_UNSET()
{
	ONEWIRE_DDR  |= (1 << ONEWIRE_DQ);
	ONEWIRE_PORT &= ~(1 << ONEWIRE_DQ);
}
//-------------------
uChar ONEWIRE_Init()
{
	ONEWIRE_UNSET();
	_delay_us(485);
	
	ONEWIRE_SET();
	_delay_us(65);
	
	if(!(ONEWIRE_PIN & (1 << ONEWIRE_DQ)))
	{
		_delay_us(245);
		
		if(!(ONEWIRE_PIN & (1 << ONEWIRE_DQ)))
			return 0;
		else
			return 1;
	}
	
	return 0;
}
//------------------------------
void ONEWIRE_WriteBit(uChar bit)
{
	ONEWIRE_UNSET(); //  
	
	if(bit)
		_delay_us(10); //   
	else
		_delay_us(65); //   
		
	ONEWIRE_SET();
	
	if(bit)
		_delay_us(55); //  -
	else
		_delay_us(5);
}
//--------------------------------
void ONEWIRE_WriteByte(uChar byte)
{
	for(uChar count = 0; count < 8; count++)
	{
		if((byte & (1 << count)) == (1 << count))
		{
			ONEWIRE_UNSET();
			_delay_us(2);
			ONEWIRE_SET();
			_delay_us(65);
		}			
		else
		{
			ONEWIRE_UNSET();
			_delay_us(65);
			ONEWIRE_SET();
			_delay_us(5);
		}
	}
}
//---------------------
uChar ONEWIRE_ReadBit()
{
	ONEWIRE_UNSET();
	_delay_us(10);
	
	return (ONEWIRE_PIN & ONEWIRE_DQ);
}
//----------------------
uChar ONEWIRE_ReadByte()
{
	uChar byte = 0;
	
	for(uChar count = 0; count < 8; count++)
	{
		ONEWIRE_UNSET();
		_delay_us(2);
		ONEWIRE_SET();
		_delay_us(7);
		
		if((ONEWIRE_PIN & (1 << ONEWIRE_DQ)) == 0x00)
			byte &= ~(1 << count);
		else
			byte |= (1 << count);
			
		_delay_us(50);
	}
	
	return byte;
}
//-------------------------------------------------------------
struct TEMP_TYPE ONEWIRE_DecodeToDecimal(uChar low, uChar high)
{	
	uChar result = (low >> 4) | (high << 4);
	
	struct TEMP_TYPE ttemp;
		
	if((result & 0x80) == 0x80)
	{
		ttemp.sign = true; //  
		ttemp.temp_int = (result ^ 0xFF) + 1;
	}
	else
	{
		ttemp.sign = false;
		ttemp.temp_int = result;
	}
	
	ttemp.temp_fract = (low & 0x0F);
	
	return ttemp;
}